@@ -46,18 +46,13 @@
 #include <libxslt/xsltInternals.h>
 #include <libxslt/xsltutils.h>
 
+#include "yelp-debug.h"
 #include "yelp-error.h"
 #include "yelp-settings.h"
 #include "yelp-toc-pager.h"
 #include "yelp-utils.h"
 #include "yelp-io-channel.h"
 
-#ifdef YELP_DEBUG
-#define d(x) x
-#else
-#define d(x)
-#endif
-
 #define YELP_NAMESPACE "http://www.gnome.org/yelp/ns"
 
 #define STYLESHEET_PATH DATADIR"/yelp/xslt/"
@@ -65,8 +60,6 @@
 
 typedef gboolean      (*ProcessFunction)        (YelpTocPager      *pager);
 
-typedef struct _YelpListing YelpListing;
-
 #define YELP_TOC_PAGER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), YELP_TYPE_TOC_PAGER, YelpTocPagerPriv))
 
 struct _YelpTocPagerPriv {
@@ -89,9 +82,12 @@ struct _YelpTocPagerPriv {
     GSList *mandir_langpath;     /* ptr to current entry in mandir_ptr */
     GHashTable *man_secthash;
     GHashTable *man_manhash;
+    GHashTable *man_dirlang;     /* key = man page directory, value = language */
 #endif
 
     xmlDocPtr     toc_doc;
+    xmlDocPtr     man_doc;
+    xmlDocPtr     info_doc;
 
     GHashTable   *unique_hash;
     GHashTable   *category_hash;
@@ -107,16 +103,6 @@ struct _YelpTocPagerPriv {
     xsltTransformContextPtr transformContext;
 };
 
-struct _YelpListing {
-    gchar       *id;
-    gchar       *title;
-
-    GSList      *listings;
-    GSList      *documents;
-
-    gboolean     has_listings;
-};
-
 static void          toc_pager_class_init      (YelpTocPagerClass *klass);
 static void          toc_pager_init            (YelpTocPager      *pager);
 static void          toc_pager_dispose         (GObject           *gobject);
@@ -257,10 +243,12 @@ yelp_toc_pager_pause (YelpTocPager *pager)
 {
     g_return_if_fail (pager != NULL);
 
-    d (g_print ("yelp_toc_pager_pause\n"));
-    d (g_print ("  pause_caunt = %i\n", pager->priv->pause_count + 1));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  pause_count = %i\n", pager->priv->pause_count + 1);
 
     pager->priv->pause_count = pager->priv->pause_count + 1;
+
+    debug_print (DB_FUNCTION, "exiting\n");
 }
 
 void
@@ -268,8 +256,8 @@ yelp_toc_pager_unpause (YelpTocPager *pager)
 {
     g_return_if_fail (pager != NULL);
 
-    d (g_print ("yelp_toc_pager_unpause\n"));
-    d (g_print ("  pause_caunt = %i\n", pager->priv->pause_count - 1));
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "  pause_count = %i\n", pager->priv->pause_count - 1);
 
     pager->priv->pause_count = pager->priv->pause_count - 1;
     if (pager->priv->pause_count < 0) {
@@ -282,6 +270,8 @@ yelp_toc_pager_unpause (YelpTocPager *pager)
 			       (GtkFunction) toc_process_pending,
 			       pager);
     }
+    
+    debug_print (DB_FUNCTION, "exiting\n");
 }
 
 /******************************************************************************/
@@ -289,7 +279,7 @@ yelp_toc_pager_unpause (YelpTocPager *pager)
 static void
 toc_pager_error (YelpPager *pager)
 {
-    d (g_print ("toc_pager_error\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     yelp_pager_set_state (pager, YELP_PAGER_STATE_ERROR);
 }
 
@@ -298,7 +288,7 @@ toc_pager_cancel (YelpPager *pager)
 {
     YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
 
-    d (g_print ("toc_pager_cancel\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     yelp_pager_set_state (pager, YELP_PAGER_STATE_INVALID);
 
     priv->cancel = TRUE;
@@ -307,7 +297,7 @@ toc_pager_cancel (YelpPager *pager)
 static void
 toc_pager_finish (YelpPager   *pager)
 {
-    d (g_print ("toc_pager_finish\n"));
+    debug_print (DB_FUNCTION, "entering\n");
     yelp_pager_set_state (pager, YELP_PAGER_STATE_FINISHED);
 }
 
@@ -316,7 +306,7 @@ toc_pager_process (YelpPager *pager)
 {
     YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
 
-    d (g_print ("toc_pager_process\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     yelp_pager_set_state (pager, YELP_PAGER_STATE_PARSING);
     g_signal_emit_by_name (pager, "parse");
@@ -333,13 +323,118 @@ toc_pager_process (YelpPager *pager)
     return FALSE;
 }
 
+/* we take advantage of the resolve_frag function to do lazy
+ * loading of each page the TOC.  frag_id is the id attribute 
+ * of the toc element that we want. 
+ *
+ * yes, this kind of a hack */
 static const gchar *
 toc_pager_resolve_frag (YelpPager *pager, const gchar *frag_id)
 {
-    if (!frag_id)
-	return "index";
-    else
+    YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
+    YelpPage *page = NULL;
+    GError *error = NULL;
+    xmlDocPtr outdoc = NULL;
+    gchar **params = NULL;
+    gint  params_i = 0;
+    gint  params_max = 10;
+    GtkIconInfo *info;
+    GtkIconTheme *theme = (GtkIconTheme *) yelp_settings_get_icon_theme ();
+
+    if (priv->toc_doc == NULL)
+	return NULL;
+
+    debug_print (DB_FUNCTION, "entering\n");
+    debug_print (DB_ARG, "frag_id = %s\n", frag_id);
+
+    /* if frag_id is NULL, we assume we are loading the top level TOC page */
+    if (frag_id == NULL)
+	frag_id = "index";
+
+    /* we use the "frag_id" as the page_id here */
+    page = yelp_pager_get_page_from_id (pager, frag_id);
+
+    /* if we found the page just return */
+    if (page != NULL) {
+	debug_print (DB_DEBUG, "found the frag_id=%s\n", frag_id);
 	return frag_id;
+    }
+
+    /* only create and parse the stylesheet on the first call to this function */
+    if (!priv->stylesheet) {
+	priv->stylesheet = xsltParseStylesheetFile (BAD_CAST TOC_STYLESHEET);
+    }
+
+    if (!priv->stylesheet) {
+	g_set_error (&error, YELP_ERROR, YELP_ERROR_PROC,
+		     _("The table of contents could not be processed. The "
+		       "file ‘%s’ is either missing or is not a valid XSLT "
+		       "stylesheet."),
+		     TOC_STYLESHEET);
+	yelp_pager_error (YELP_PAGER (pager), error);
+	goto done;
+    }
+
+    priv->transformContext = xsltNewTransformContext (priv->stylesheet,
+						      priv->toc_doc);
+    priv->transformContext->_private = pager;
+    xsltRegisterExtElement (priv->transformContext,
+			    BAD_CAST "document",
+			    BAD_CAST YELP_NAMESPACE,
+			    (xsltTransformFunction) xslt_yelp_document);
+
+    params = g_new0 (gchar *, params_max);
+    yelp_settings_params (&params, &params_i, &params_max);
+
+    if ((params_i + 10) >= params_max - 1) {
+	params_max += 10;
+	params = g_renew (gchar *, params, params_max);
+    }
+
+    info = gtk_icon_theme_lookup_icon (theme, "yelp-icon-big", 192, 0);
+    if (info) {
+	params[params_i++] = "help_icon";
+	params[params_i++] = g_strdup_printf ("\"%s\"",
+					      gtk_icon_info_get_filename (info));
+	params[params_i++] = "help_icon_size";
+	params[params_i++] = g_strdup_printf ("%i",
+					      gtk_icon_info_get_base_size (info));
+	gtk_icon_info_free (info);
+    }
+
+    /* we specify the id attribute of the toc element that we want to process to
+     * the stylesheet */
+    params[params_i++] = "yelp.toc.id";
+    params[params_i++] = g_strdup_printf ("\'%s\'", frag_id);
+
+    params[params_i++] = NULL;
+
+    outdoc = xsltApplyStylesheetUser (priv->stylesheet,
+				      priv->toc_doc,
+				      (const gchar **)params, NULL, NULL,
+				      priv->transformContext);
+    /* Don't do this */
+    g_signal_emit_by_name (pager, "finish");
+
+ done:
+    if (params) {
+	for (params_i = 0; params[params_i] != NULL; params_i++)
+	    if (params_i % 2 == 1)
+		g_free ((gchar *) params[params_i]);
+    }
+    if (outdoc)
+	xmlFreeDoc (outdoc);
+    /* with this hack (this function) we can't free the toc_doc */
+    /*if (priv->toc_doc) {
+	xmlFreeDoc (priv->toc_doc);
+	priv->toc_doc = NULL;
+    }*/
+    if (priv->transformContext) {
+	xsltFreeTransformContext (priv->transformContext);
+	priv->transformContext = NULL;
+    }
+
+    return frag_id;
 }
 
 static GtkTreeModel *
@@ -358,15 +453,13 @@ toc_process_pending (YelpTocPager *pager)
     static ProcessFunction process_funcs[] = {
 	process_read_menu,
 	process_omf_pending,
-	process_xslt,
 #ifdef ENABLE_MAN
 	process_mandir_pending,
-	process_xslt,
 #endif
 #ifdef ENABLE_INFO
 	process_info_pending,
-	process_xslt,
 #endif
+	/* process_xslt, */
 	process_cleanup,
 	NULL
     };
@@ -990,6 +1083,14 @@ add_man_page_to_toc (YelpTocPager *pager, gchar *dirname, gchar *filename)
 	    info = yelp_doc_info_get (url_full, TRUE);
 
 	    if (info) {
+		gchar *lang = NULL;
+
+		if (priv->man_dirlang)
+		    lang = g_hash_table_lookup (priv->man_dirlang, dirname);
+
+		if (lang)
+		    yelp_doc_info_set_language (info, lang);
+
 		yelp_doc_info_add_uri (info, url_short, YELP_URI_TYPE_MAN);
 		tmp = xmlNewChild (tmp, NULL, BAD_CAST "doc", NULL);
 		xmlNewProp (tmp, BAD_CAST "href", BAD_CAST url_full);
@@ -1050,11 +1151,14 @@ create_manindex_file (gchar *index_file, xmlDocPtr xmldoc)
 static int
 create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 {
+    const gchar * const * langs = g_get_language_names ();
     xmlXPathContextPtr xpath = NULL;
     xmlXPathObjectPtr objsect = NULL;
     xmlDocPtr manindex_xml = NULL;
+    xmlNodePtr root = NULL;
+    xmlChar *language = NULL;
     gint update_flag = 0;
-    gint i, j, k;
+    gint i, j;
     
     YelpTocPagerPriv *priv  = YELP_TOC_PAGER (pager)->priv;
 
@@ -1066,6 +1170,23 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 	return 0;
     }
     
+    /* check the language that this index file was generated for */
+    root = xmlDocGetRootElement (manindex_xml);
+    language = xmlGetProp (root, BAD_CAST "lang");
+    
+    /* if the language is not the same as the current language (specified
+     * by the LANGUAGE environment variable) then return so that the index 
+     * is recreated */
+    if (language == NULL || !g_str_equal (BAD_CAST language, langs[0])) {
+	debug_print (DB_INFO, 
+	             "LANGUAGE and index file language do not match, "
+		     "index file will be recreated.\n");
+	xmlFreeDoc (manindex_xml);
+	return 0;
+    }
+
+    xmlFree (language);
+    
     xpath = xmlXPathNewContext (manindex_xml);
     objsect = xmlXPathEvalExpression (BAD_CAST "/manindex/mansect", xpath);
 
@@ -1080,28 +1201,29 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 	objdirs = xmlXPathEvalExpression (BAD_CAST "dir", xpath);
 
 	for (j=0; j < objdirs->nodesetval->nodeNr; j++) {
-	    xmlXPathObjectPtr objdirname;
-	    xmlXPathObjectPtr objdirmtime;
-	    xmlXPathObjectPtr objmanpages;
 	    xmlNodePtr dirnode = objdirs->nodesetval->nodeTab[j];
 	    xmlNodePtr node = NULL;
 	    xmlChar *dirname = NULL;
 	    xmlChar *dirmtime = NULL;
+	    xmlChar *lang = NULL;
 	    time_t mtime;
 	    struct stat buf;
 	    
-	    xpath->node = dirnode;
-	    objdirmtime = xmlXPathEvalExpression (BAD_CAST "@mtime", xpath);
-	    
-	    node = objdirmtime->nodesetval->nodeTab[0];
-	    dirmtime = xmlNodeListGetString (manindex_xml, 
-	                                     node->xmlChildrenNode, 1);
-	    
-	    objdirname = xmlXPathEvalExpression (BAD_CAST "name[1]", xpath);
+            lang     = xmlGetProp (dirnode, BAD_CAST "lang");
+	    dirmtime = xmlGetProp (dirnode, BAD_CAST "mtime");
 
-	    node = objdirname->nodesetval->nodeTab[0];
-	    dirname = xmlNodeListGetString (manindex_xml, 
-	                                    node->xmlChildrenNode, 1);
+	    if (lang == NULL)
+		lang = xmlStrdup (BAD_CAST "C");
+
+	    for (node = dirnode->children; node != NULL; node = node->next) {
+		if (node->type == XML_ELEMENT_NODE && 
+		    g_str_equal ((gchar *)node->name, "name")) {
+		    dirname = xmlNodeGetContent (node);
+		}
+	    }
+
+	    g_hash_table_insert (priv->man_dirlang, g_strdup ((gchar *)dirname), 
+	                         g_strdup ((gchar *)lang));
 	    
 	    /* if we can't stat the dirname for some reason, then skip adding
 	     * this directory to the TOC, remove the dirnode, and set the flag
@@ -1155,25 +1277,22 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 	    /* otherwise just read from the index file */
 	    } else {
 	    
-		objmanpages = xmlXPathEvalExpression (BAD_CAST "page", xpath);
-
-		for (k=0; k < objmanpages->nodesetval->nodeNr; k++) {
-		    xmlNodePtr node = objmanpages->nodesetval->nodeTab[k];
+		for (node = dirnode->children; node != NULL; node = node->next) {
 		    xmlChar *manpage = NULL;
 
-		    manpage = xmlNodeListGetString (manindex_xml,
-		                                    node->xmlChildrenNode, 1);
-
-		    add_man_page_to_toc (pager, (gchar *)dirname, (gchar *)manpage);
-		    priv->manpage_count++;
-		    xmlFree (manpage);
-	        }
-		xmlXPathFreeObject (objmanpages);
+		    if (node->type == XML_ELEMENT_NODE && 
+		        g_str_equal ((gchar *)node->name, "page")) {
+			manpage = xmlNodeGetContent (node);
+			add_man_page_to_toc (pager, (gchar *)dirname, (gchar *)manpage);
+			priv->manpage_count++;
+			xmlFree (manpage);
+		    }
+		}
+	    
 	    }
+	    xmlFree (lang);
 	    xmlFree (dirmtime);
 	    xmlFree (dirname);
-	    xmlXPathFreeObject (objdirmtime);
-	    xmlXPathFreeObject (objdirname);
 	}
 
 	/* cleanup */
@@ -1191,7 +1310,7 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 
     xmlXPathFreeContext (xpath);
     xmlXPathFreeObject (objsect);
-    xmlFree (manindex_xml);
+    xmlFreeDoc (manindex_xml);
     
     return 1;
 }
@@ -1207,18 +1326,18 @@ process_mandir_pending (YelpTocPager *pager)
 
     YelpTocPagerPriv *priv  = YELP_TOC_PAGER (pager)->priv;
 
-    if (!priv->toc_doc) {
+    if (!priv->man_doc) {
 	xmlXPathContextPtr xpath;
 	xmlXPathObjectPtr  obj;
 
 	/* NOTE: this document is free()'d at the end of the process_xslt function */
-	priv->toc_doc = xmlCtxtReadFile (priv->parser, DATADIR "/yelp/man.xml", NULL,
+	priv->man_doc = xmlCtxtReadFile (priv->parser, DATADIR "/yelp/man.xml", NULL,
 					 XML_PARSE_NOBLANKS | XML_PARSE_NOCDATA  |
 					 XML_PARSE_NOENT    | XML_PARSE_NOERROR  |
 					 XML_PARSE_NONET    );
-	priv->man_secthash = g_hash_table_new (g_str_hash, g_str_equal);
+	priv->man_secthash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
-	xpath = xmlXPathNewContext (priv->toc_doc);
+	xpath = xmlXPathNewContext (priv->man_doc);
 	obj = xmlXPathEvalExpression (BAD_CAST "//toc", xpath);
 
 	for (i = 0; i < obj->nodesetval->nodeNr; i++) {
@@ -1229,11 +1348,14 @@ process_mandir_pending (YelpTocPager *pager)
 		gchar **sects = g_strsplit ((gchar *)sect, " ", 0);
 
 		for (j = 0; sects[j] != NULL; j++)
-		    g_hash_table_insert (priv->man_secthash, sects[j], node);
+		    g_hash_table_insert (priv->man_secthash, 
+					 g_strdup (sects[j]), node);
+		g_strfreev (sects);
 	    }
-
+	    xmlFree (sect);
 	    xml_trim_titles (node);
 	}
+	xmlXPathFreeObject (obj);
 	xmlXPathFreeContext (xpath);
     }
 
@@ -1251,11 +1373,21 @@ process_mandir_pending (YelpTocPager *pager)
 	gchar **manpaths = NULL;
 	gchar **mandirs = NULL;
 
+	if (!priv->man_dirlang)
+	    priv->man_dirlang = g_hash_table_new_full (g_str_hash, g_str_equal,
+	                                               g_free,     g_free);
+
 	/* check for the existence of the xml cache file in ~/.gnome2/yelp.d/
 	 * if it exists, use it as the source for man pages instead of 
 	 * searching the hard disk for them - should make startup much faster */
 	if (g_file_test (index_file, G_FILE_TEST_IS_REGULAR) &&
 	    create_toc_from_index (pager, index_file)) {
+	    xmlNodePtr mynode;
+
+            mynode = xmlCopyNode (xmlDocGetRootElement (priv->man_doc), 1);
+	    xmlAddChild (xmlDocGetRootElement(priv->toc_doc), mynode);
+
+	    xmlFreeDoc (priv->man_doc);
 
 	    /* we are done processing, return FALSE so we don't call this 
 	     * function again. */
@@ -1265,6 +1397,7 @@ process_mandir_pending (YelpTocPager *pager)
 	mandirs = yelp_get_man_paths ();
 	priv->manindex_xml = xmlNewDoc (BAD_CAST "1.0");
 	priv->root = xmlNewNode (NULL, BAD_CAST "manindex");
+	xmlNewProp (priv->root, BAD_CAST "lang", BAD_CAST langs[0]);
 	priv->ins  = priv->root;
 
 	xmlDocSetRootElement (priv->manindex_xml, priv->root);
@@ -1282,6 +1415,33 @@ process_mandir_pending (YelpTocPager *pager)
 	manpaths = g_strsplit (manpath, G_SEARCHPATH_SEPARATOR_S, -1);
 	g_free (manpath);
 
+	/* order is important here, since we may encounter collisions when adding 
+	 * a man page to the hash table "man_manhash".
+	 *
+	 * We want the resulting list to be sorted in order of priority:
+	 * 1) Highest priority is given to the base directory name returned 
+	 *    from $MANPATH or the manpath program.  So if that is 
+	 *    /usr/local/share:/usr/share then the man page 
+	 *    /usr/local/share/man/man1/python.1.gz would have precedence over
+	 *    /usr/share/man/man1/python.1.gz 
+	 * 2) Next highest priority is given to the language.
+	 * 3) Lowest priority is given to the section name, i.e. "man0p" "man1"
+	 *    since a man page from one section should never conflict with a man page
+	 *    from another section (because of the .0p or .1 section in the filename)
+	 *
+	 * Here is an example of how it should be sorted when 
+	 *   $MANPATH=/usr/local/share:/usr/share and LANGUAGE="es"
+	 *   
+	 * Adding /usr/local/share/man/es/man0p
+	 * Adding /usr/share/man/es/man0p
+	 * Adding /usr/local/share/man/man0p
+	 * Adding /usr/share/man/man0p
+	 * Adding /usr/local/share/man/es/man1
+	 * Adding /usr/share/man/es/man1
+	 * Adding /usr/local/share/man/man1
+	 * Adding /usr/share/man/man1 
+	 */
+	
 	for (i=0; mandirs[i] != NULL; i++) {
 	    GSList *tmplist = NULL;
 
@@ -1294,12 +1454,35 @@ process_mandir_pending (YelpTocPager *pager)
 			                            mandirs[i],
 			                            NULL);
 
+		    g_hash_table_insert (priv->man_dirlang, g_strdup (dirname),
+		                         g_strdup (langs[j]));
+
+		    /* prepend to list for speed, reverse it at the end */
 		    tmplist = g_slist_prepend (tmplist, dirname);
 		}
 	    }
+	    tmplist = g_slist_reverse (tmplist);
 
 	    priv->mandir_list = g_slist_prepend (priv->mandir_list, tmplist);
 	}
+	priv->mandir_list = g_slist_reverse (priv->mandir_list);
+
+#ifdef YELP_DEBUG
+	/* debugging: print out lists in order */
+	GSList *list1 = priv->mandir_list;
+	GSList *list2 = NULL;
+
+	while (list1 != NULL) {
+	    list2 = list1->data;
+	    while (list2 && list2->data) {
+	        debug_print (DB_INFO, "Dir=%s\n", (gchar *)list2->data);
+	        list2 = g_slist_next (list2);
+	    }
+	    list1 = g_slist_next (list1);
+	}
+#endif
+
+	g_strfreev (manpaths);
  
 	priv->mandir_ptr = priv->mandir_list;
 	if (priv->mandir_list && priv->mandir_list->data) {
@@ -1318,9 +1501,10 @@ process_mandir_pending (YelpTocPager *pager)
 	if ((dir = g_dir_open (dirname, 0, NULL))) {
 	    struct stat buf;
 	    gchar mtime_str[20];
+	    gchar *lang = NULL;
 
 	    if (g_stat (dirname, &buf) < 0)
-		g_warning ("Unable to stat dir: \"%s\"\n", dirname);
+		debug_print (DB_WARN, "Unable to stat dir: \"%s\"\n", dirname);
 
 	    /* check to see if we need create a new hash table */
 	    if (!priv->man_manhash) {
@@ -1334,7 +1518,17 @@ process_mandir_pending (YelpTocPager *pager)
 
 	    g_snprintf (mtime_str, 20, "%u", (guint) buf.st_mtime);
 
+	    lang = g_hash_table_lookup (priv->man_dirlang, dirname);
+
 	    priv->ins = xmlNewChild (priv->ins, NULL, BAD_CAST "dir", NULL);
+
+	    if (lang == NULL) {
+		g_print ("dir %s is null\n", dirname);
+		lang = "C";
+	    }
+	    
+	    xmlNewProp (priv->ins, BAD_CAST "lang", BAD_CAST lang);
+
 	    xmlNewProp (priv->ins, BAD_CAST "mtime", BAD_CAST mtime_str);
 	    xmlAddChild (priv->ins, xmlNewText (BAD_CAST "\n      "));
 	    xmlNewChild (priv->ins, NULL, BAD_CAST "name", BAD_CAST dirname);
@@ -1369,6 +1563,7 @@ process_mandir_pending (YelpTocPager *pager)
 	    }
 	} else {   /* no more entries to prcoess, write file & cleanup */
 	    GSList *listptr = priv->mandir_list;
+	    xmlNodePtr copynode;
 
 	    while (listptr && listptr->data)  {
 		GSList *langptr = listptr->data;
@@ -1389,6 +1584,11 @@ process_mandir_pending (YelpTocPager *pager)
 
 	    xmlFree (priv->manindex_xml);
 
+            copynode = xmlCopyNode (xmlDocGetRootElement (priv->man_doc), 1);
+	    xmlAddChild (xmlDocGetRootElement(priv->toc_doc), copynode);
+
+	    xmlFreeDoc (priv->man_doc);
+
 	    /* done processing */
 	    return FALSE;
 	}
@@ -1405,6 +1605,7 @@ process_info_pending (YelpTocPager *pager)
     gchar ** info_paths = yelp_get_info_paths ();
     int i = 0;
     gchar *filename = NULL;
+    xmlNodePtr mynode = NULL;
 
     GHashTable *info_parsed = g_hash_table_new_full (g_str_hash, g_str_equal,
 						     g_free, g_free);
@@ -1413,7 +1614,6 @@ process_info_pending (YelpTocPager *pager)
     
     YelpTocPagerPriv *priv = YELP_TOC_PAGER (pager)->priv;
     xmlNodePtr node = NULL;
-    
 
     for (i=0; info_paths[i]; i++) {
 	filename = g_strconcat (info_paths[i], "/dir", NULL);
@@ -1431,24 +1631,32 @@ process_info_pending (YelpTocPager *pager)
 	    xmlNodePtr tmp;
 	    xmlNodePtr new_node = NULL;
 	    gboolean menufound = FALSE;
-	    
-	    if (!priv->toc_doc) {
+	    gchar ** amp;
+
+	    if (!priv->info_doc) {
 		xmlXPathContextPtr xpath;
 		xmlXPathObjectPtr  obj;
-		priv->toc_doc = xmlCtxtReadFile (priv->parser, DATADIR "/yelp/info.xml", NULL,
+		priv->info_doc = xmlCtxtReadFile (priv->parser, DATADIR "/yelp/info.xml", NULL,
 						 XML_PARSE_NOBLANKS | XML_PARSE_NOCDATA  |
 						 XML_PARSE_NOENT    | XML_PARSE_NOERROR  |
 						 XML_PARSE_NONET    );
 		
-		xpath = xmlXPathNewContext (priv->toc_doc);
+		xpath = xmlXPathNewContext (priv->info_doc);
 		obj = xmlXPathEvalExpression (BAD_CAST "//toc", xpath);
 		node = obj->nodesetval->nodeTab[0];
+		for (i=0; i < obj->nodesetval->nodeNr; i++) {
+		    xmlNodePtr tmpnode = obj->nodesetval->nodeTab[i];
+		    xml_trim_titles (tmpnode);
+		}
+		xmlXPathFreeObject (obj);
+		xmlXPathFreeContext (xpath);
 	    }
 	    
 	    channel = yelp_io_channel_new_file (filename, NULL);
 	    g_io_channel_read_to_end (channel, &str, &len, NULL);
 	    g_io_channel_shutdown (channel, FALSE, NULL);
-	    
+	    g_io_channel_unref (channel);
+
 	    files = g_strsplit (str, "\n", -1);
 	    
 	    for (ptr = files; *ptr != NULL; ptr++) {
@@ -1480,6 +1688,7 @@ process_info_pending (YelpTocPager *pager)
 			p = g_strdup (part2);
 			p = g_strstrip (p);
 			tooltip = g_strdup_printf (_("Read info page for %s"), p);
+			g_free (p);
 			path = part1+1;
 			part1 = strchr (path, ')');
 			if (!part1)
@@ -1515,12 +1724,12 @@ process_info_pending (YelpTocPager *pager)
 					       NULL);
 			    xmlNewNsProp (tmp, NULL, BAD_CAST "href", BAD_CAST fname);
 			    
-			    xmlNewChild (tmp, NULL, BAD_CAST "title",
+			    xmlNewTextChild (tmp, NULL, BAD_CAST "title",
 					 BAD_CAST part2);
-			    xmlNewChild (tmp, NULL, BAD_CAST "tooltip",
+			    xmlNewTextChild (tmp, NULL, BAD_CAST "tooltip",
 					 BAD_CAST tooltip);
 			    
-			    xmlNewChild (tmp, NULL, BAD_CAST "description",
+			    xmlNewTextChild (tmp, NULL, BAD_CAST "description",
 					 BAD_CAST desc);
 			}
 		    done:
@@ -1532,18 +1741,23 @@ process_info_pending (YelpTocPager *pager)
 		    } else if (!g_ascii_isspace (**ptr) && g_ascii_isprint(**ptr)){
 			new_node = g_hash_table_lookup (categories, *ptr);
 			if (!new_node) {
+			    gchar *tmp;
 
 			    /* A new section */
 			    static int sectno = 1;
 			    new_node = xmlNewChild (node, NULL, BAD_CAST "toc",
 						    NULL);
+			    tmp = g_strdup_printf ("%d", sectno);
 			    xmlNewNsProp (new_node, NULL, BAD_CAST "sect",
-					  BAD_CAST g_strdup_printf ("%d", sectno));
+					  BAD_CAST tmp);
+			    g_free (tmp);
+			    tmp = g_strdup_printf ("infosect%d", sectno);
 			    xmlNewNsProp (new_node, NULL, BAD_CAST "id",
-					  BAD_CAST g_strdup_printf ("infosect%d", sectno));
+					  BAD_CAST tmp);
+			    g_free (tmp);
 			    sectno++;
-			    xmlNewChild (new_node, NULL, BAD_CAST "title",
-					 BAD_CAST *ptr);
+			    xmlNewTextChild (new_node, NULL, BAD_CAST "title",
+					     BAD_CAST *ptr);
 			    g_hash_table_insert (categories, 
 						 g_strdup (*ptr),
 						 new_node);
@@ -1559,6 +1773,12 @@ process_info_pending (YelpTocPager *pager)
     }
     g_hash_table_destroy (info_parsed);
     g_hash_table_destroy (categories);
+    
+    mynode = xmlCopyNode (xmlDocGetRootElement (priv->info_doc), 1);
+    xmlAddChild (xmlDocGetRootElement (priv->toc_doc), mynode);
+    
+    xmlFreeDoc (priv->info_doc);
+    
     return FALSE;
 }
 #endif /* ENABLE_INFO */
@@ -1593,7 +1813,7 @@ process_read_menu (YelpTocPager *pager)
     for (i = 0; i < obj->nodesetval->nodeNr; i++) {
 	xmlNodePtr node = obj->nodesetval->nodeTab[i];
 	xmlChar *icon = NULL;
-#ifdef ENABLE_MAN_OR_INFO
+#if 0
 	xmlChar *id = NULL;
 	xmlNodePtr cmdhelp;
 	xmlNodePtr newnode;
@@ -1789,12 +2009,35 @@ process_cleanup (YelpTocPager *pager)
 {
     YelpTocPagerPriv *priv = pager->priv;
 
+    /* dump the toc_doc if we are debugging */
+    /*d (xmlDocDump (stdout, priv->toc_doc)); */
+
+#ifdef ENABLE_MAN
+    /* clean up the man directory language hash table */
+    if (priv->man_dirlang) {
+        g_hash_table_destroy (priv->man_dirlang);
+        priv->man_dirlang = NULL;
+    }
+
+    /* clean up the man page section hash table */
+    if (priv->man_secthash) {
+	g_hash_table_destroy (priv->man_secthash);
+	priv->man_secthash = NULL;
+    }
+#endif
+
     /* cleanup the stylesheet used to process the toc */
     if (priv->stylesheet) {
 	xsltFreeStylesheet (priv->stylesheet);
 	priv->stylesheet = NULL;
     }
 
+    /* cleanup the parser context */
+    if (priv->parser) {
+	xmlFreeParserCtxt (priv->parser);
+	priv->parser = NULL;
+    }
+
     /* we only ever want to run this function once, so always return false */
     return FALSE;
 }
